//
//  NSURLRequest+Server.m
//  SocketServer
//
//  Created by Clemens Wagner on 05.09.15.
//  Copyright © 2015 Clemens Wagner. All rights reserved.
//

#import "NSURLRequest+Server.h"

@implementation NSURLRequest (Server)

+ (instancetype)requestWithData:(NSData *)inData bodyOffset:(NSUInteger *)outOffset {
    return [[NSMutableURLRequest requestWithData:inData bodyOffset:outOffset] copy];
}

@end

@implementation NSMutableURLRequest (Server)

+ (instancetype)requestWithData:(NSData *)inData bodyOffset:(NSUInteger *)outOffset {
    NSUInteger theLength = inData.length;
    const char *theBytes = inData.bytes;
    const char *theSeparator = strnstr(theBytes, "\r\n\r\n", theLength);
    
    if(theSeparator == NULL) {
        return nil;
    }
    else {
        NSUInteger theHeaderLength = theSeparator - theBytes;
        NSString *theHeader = [[NSString alloc] initWithData:[inData subdataWithRange:NSMakeRange(0, theHeaderLength)] encoding:NSUTF8StringEncoding];
        NSMutableURLRequest *theRequest = [self new];
        NSArray *theLines = [theHeader componentsSeparatedByString:@"\r\n"];
        NSString *thePath = nil;
        NSString *theHost = nil;
        
        *outOffset = theHeaderLength + 4;
        for(NSString *theLine in theLines) {
            if(thePath == nil) {
                NSArray *theParts = [theLine componentsSeparatedByString:@" "];
                
                theRequest.HTTPMethod = theParts.firstObject;
                thePath = theParts[1];
            }
            else {
                NSRange theRange = [theLine rangeOfString:@": "];
                
                if(theRange.location != NSNotFound) {
                    NSString *theName = [theLine substringToIndex:theRange.location];
                    NSString *theValue = [theLine substringFromIndex:theRange.location + theRange.length];
                    
                    [theRequest addValue:theValue forHTTPHeaderField:theName];
                }
            }
        }
        theHost = [theRequest valueForHTTPHeaderField:@"Host"];
        if(thePath != nil && theHost != nil) {
            NSString *theURL = [NSString stringWithFormat:@"http://%@%@", theHost, thePath];
            
            theRequest.URL = [NSURL URLWithString:theURL];
        }
        return theRequest.URL == nil ? nil : theRequest;
    }
}

@end
