//
//  ViewController.m
//  UIKit
//
//  Created by Clemens Wagner on 08.08.15.
//  Copyright © 2015 Clemens Wagner. All rights reserved.
//

#import "ViewController.h"
#import "NSString+Extensions.h"

static NSString * const kHeader = @"<style>form { display: none; height: 0; top: 0; } form input { position: absolute; height: 0; top: 0; }</style>"
@"<script type='text/javascript'>var originalSearch = search; search = function(inTerm) { originalSearch(inTerm); window.location.href = \"event://search?\" + $.param({ term: inTerm }); }</script>";

@interface ViewController ()<UITextFieldDelegate, UIWebViewDelegate>

@property (weak, nonatomic) IBOutlet UIWebView *webView;
@property (weak, nonatomic) IBOutlet UITextField *queryField;
@property (nonatomic) BOOL loadDirect;

@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    NSURL *theURL = [NSURL URLWithString:@"http://localhost:8080/"];
    NSURLRequest *theRequest = [NSURLRequest requestWithURL:theURL];
                   
    [self loadRequest:theRequest];
}

- (void)loadRequest:(NSURLRequest *)inRequest {
    NSURLSession *theSession = [NSURLSession sharedSession];
    NSURLSessionTask *theTask = [theSession dataTaskWithRequest:inRequest completionHandler:^(NSData *inData, NSURLResponse * inResponse, NSError *inError) {
        if(inData == nil) {
            NSLog(@"loadRequest: failed");
        }
        else {
            NSString *theTextEncoding = inResponse.textEncodingName;
            NSStringEncoding theEncoding = theTextEncoding == nil ? NSUTF8StringEncoding : [theTextEncoding stringEncoding];
            NSString *theContent = [[NSString alloc] initWithData:inData encoding:theEncoding];
            NSRange theRange = [theContent rangeOfString:@"</head>"];
            
            theContent = [theContent stringByReplacingCharactersInRange:NSMakeRange(theRange.location, 0) withString:kHeader];
            self.loadDirect = YES;
            [self.webView loadHTMLString:theContent baseURL:inRequest.URL];
        }
    }];
    
    [theTask resume];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark UITextFieldDelegate

- (void)textFieldDidBeginEditing:(UITextField *)inTextField {
    [self.webView.scrollView setContentOffset:CGPointZero animated:YES];
    [self.webView stringByEvaluatingJavaScriptFromString:@"$('#search input[name=q]').keydown().keyup();"];
}

- (BOOL)textField:(UITextField *)inTextField shouldChangeCharactersInRange:(NSRange)inRange replacementString:(NSString *)inText {
    if([inText isEqualToString:@"\n"]) {
        [self.webView stringByEvaluatingJavaScriptFromString:@"document.forms.search.submit();"];
        [inTextField endEditing:YES];
    }
    else {
        NSString *theText = [inTextField.text stringByReplacingCharactersInRange:inRange withString:inText];
        NSString *theScript = [NSString stringWithFormat:@"$('#search input[name=q]').val(%@).keydown().keyup();",
                               theText.javaScriptString];
        
        [self.webView stringByEvaluatingJavaScriptFromString:theScript];
    }
    return YES;
}

#pragma mark UIWebViewDelegate

- (BOOL)webView:(UIWebView *)inWebView shouldStartLoadWithRequest:(NSURLRequest *)inRequest navigationType:(UIWebViewNavigationType)inNavigationType {
    NSURL *theURL = inRequest.URL;
    
    if([theURL.scheme isEqualToString:@"event"]) {
        NSURLComponents *theComponents = [NSURLComponents componentsWithURL:theURL resolvingAgainstBaseURL:YES];
        
        for(NSURLQueryItem *theItem in theComponents.queryItems) {
            if([theItem.name isEqualToString:@"term"]) {
                self.queryField.text = theItem.value;
            }
        }
        return NO;
    }
    else if(self.loadDirect) {
        self.loadDirect = NO;
        return YES;
    }
    else {
        [self loadRequest:inRequest];
        return NO;
    }
}

- (void)webViewDidFinishLoad:(UIWebView *)inWebView {
    [inWebView stringByEvaluatingJavaScriptFromString:@"$('#search input[name=q]').autocomplete({ open: function(event, ui) { $('.ui-autocomplete').off('menufocus hover mouseover mouseenter');}});"];
}

@end
